/*
 * Turn on MASC tty discipline on a given tty
 */

#include <termios.h>
#include <sys/ioctl.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <signal.h>

#ifndef N_MASC
#define N_MASC 8
#endif

static int fd = -1;			/* terminal file descriptor	*/
static char *progname;
static char *ttname;

static void pexit(char *msg)
{
	fprintf(stderr, "%s: ", progname);
	perror(msg);
	exit(EXIT_FAILURE);
}

static void set_discipline(int which)
{
	int mascdisc = which;
	if (ioctl(fd, TIOCSETD, &mascdisc) < 0)
		pexit("ioctl(TIOCSETD)");
}

void interrupt(int sig)
{

}

int main(int argc, char **argv)
{
	sigset_t blocked;

	if (argc != 2 && argv[0]) {
		fprintf(stderr, "usage: %s <tty_name>\n", argv[0]);
		exit(EXIT_FAILURE);
	}

	progname = argv[0];
	ttname = argv[1];

	fd = open(ttname, O_RDWR | O_NOCTTY);

	if (fd == -1)
		pexit(ttname);

	signal(SIGTERM, interrupt);
	signal(SIGINT, interrupt);

	set_discipline(N_MASC);

	/*
	 * we must keep the tty open
	 */

	sigemptyset(&blocked);
	sigsuspend(&blocked);

	set_discipline(N_TTY);

	return 0;
}
